# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    GetJobStatusResponse,
    GetJobStatusResponseFormatOpts,
    GetJobStatusResponseFormatOptsFormat,
    GetJobStatusResponseOutput,
    GetJobStatusResponseOutputData,
    GetJobStatusResponseOutputMetadata,
    InitializeRenderJobRequestFormatOpts,
    InitializeRenderJobRequestFormatOptsFormat,
    InitializeRenderJobRequestTarget,
    InitializeRenderJobRequestType,
    InitializeRenderJobResponse,
    InitializeRenderJobResponseTarget,
    StartImmediateRenderRequestFormatOpts,
    StartImmediateRenderRequestFormatOptsFormat,
    StartImmediateRenderRequestTarget,
    StartImmediateRenderRequestType,
    StartImmediateRenderResponse,
    StartRenderJobResponse,
    StartRenderJobResponseFormatOpts,
    StartRenderJobResponseFormatOptsFormat,
    StartRenderJobResponseOutput,
    StartRenderJobResponseOutputData,
    StartRenderJobResponseOutputMetadata,
    StartRenderJobResponseTarget,
)

__all__ = [
    "GetJobStatusResponse",
    "GetJobStatusResponseFormatOpts",
    "GetJobStatusResponseFormatOptsFormat",
    "GetJobStatusResponseOutput",
    "GetJobStatusResponseOutputData",
    "GetJobStatusResponseOutputMetadata",
    "InitializeRenderJobRequestFormatOpts",
    "InitializeRenderJobRequestFormatOptsFormat",
    "InitializeRenderJobRequestTarget",
    "InitializeRenderJobRequestType",
    "InitializeRenderJobResponse",
    "InitializeRenderJobResponseTarget",
    "StartImmediateRenderRequestFormatOpts",
    "StartImmediateRenderRequestFormatOptsFormat",
    "StartImmediateRenderRequestTarget",
    "StartImmediateRenderRequestType",
    "StartImmediateRenderResponse",
    "StartRenderJobResponse",
    "StartRenderJobResponseFormatOpts",
    "StartRenderJobResponseFormatOptsFormat",
    "StartRenderJobResponseOutput",
    "StartRenderJobResponseOutputData",
    "StartRenderJobResponseOutputMetadata",
    "StartRenderJobResponseTarget",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .generate_template_previews_response_pdf_preview import GenerateTemplatePreviewsResponsePdfPreview
from .generate_template_previews_response_png_preview import GenerateTemplatePreviewsResponsePngPreview


class GenerateTemplatePreviewsResponse(UniversalBaseModel):
    png_preview: typing_extensions.Annotated[
        GenerateTemplatePreviewsResponsePngPreview, FieldMetadata(alias="pngPreview")
    ]
    pdf_preview: typing_extensions.Annotated[
        GenerateTemplatePreviewsResponsePdfPreview, FieldMetadata(alias="pdfPreview")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .generate_template_previews_request_format_opts_format import GenerateTemplatePreviewsRequestFormatOptsFormat


class GenerateTemplatePreviewsRequestFormatOpts(UniversalBaseModel):
    """
    Format options for the rendered document
    """

    from_page: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="fromPage")] = None
    to_page: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="toPage")] = None
    format: typing.Optional[GenerateTemplatePreviewsRequestFormatOptsFormat] = None
    wait_for_selector: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="waitForSelector")] = (
        pydantic.Field(default=None)
    )
    """
    Selector to wait for to know when the page is loaded and can be saved as pdf, png, etc.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

StartImmediateRenderRequestType = typing.Union[
    typing.Literal["docx", "xlsx", "pptx", "ejs", "html", "latex", "react"], typing.Any
]



import json
import os
from pogodoc import PogodocClient, RenderConfig, UpdateTemplateRequestTemplateInfo, SaveCreatedTemplateRequestTemplateInfo, InitializeRenderJobRequestFormatOpts
from dotenv import load_dotenv

load_dotenv()

def readJson(path: str):
    with open(path, "r") as f:
        return json.load(f)
    
sampleData = readJson("../../data/json_data/react.json")
templatePath = "../../data/templates/React-Demo-App.zip"

def main():
    client = PogodocClient(token=os.getenv("POGODOC_API_TOKEN"))

    test_readme_example()

    test_document_generations(client)

    # save template
    templateId = client.save_template(
        path=templatePath, 
        template_info=SaveCreatedTemplateRequestTemplateInfo(
            title="Test Template",
            description="Test Description", 
            type="html",
            sample_data=sampleData, 
            categories=["invoice"]
        )
    )

    # generate document
    document = client.generate_document(template_id=templateId, data=sampleData, render_config=RenderConfig(type="html", target="pdf"))
    print(document)

    # get template index html
    templateHtml = client.templates.get_template_index_html(template_id=templateId)
    print(templateHtml)

    # update template
    contentId = client.update_template(template_id=templateId, path=templatePath, template_info=UpdateTemplateRequestTemplateInfo(title="Test Template", description="Test Description", type="html", sample_data=sampleData, categories=["invoice"]))
    print(contentId)

    # generate presigned url
    presignedUrl = client.templates.generate_presigned_get_url(template_id=templateId)
    print(presignedUrl)

    # immediate render with template string
    immediateRender = client.documents.start_immediate_render(template="<h1>Hello <%= name %></h1>", data={"name": "John Doe"}, target="pdf", type="html")
    print(immediateRender)

    # # delete template
    # client.templates.delete_template(template_id=templateId)

def test_readme_example():
    client = PogodocClient(
        token=os.getenv("POGODOC_API_TOKEN"),
    )

    response = client.generate_document(
        template_id = os.getenv("TEMPLATE_ID"),
        data = {"name": "John Doe"},
        render_config = RenderConfig(
            type = "html",
            target = "pdf",
            format_opts = InitializeRenderJobRequestFormatOpts(
                from_page = 1,
            ),
        ),
    )

    print("README Generated document url:\n", response.output.data.url)

def test_document_generations(client: PogodocClient):
    template_id = os.getenv("TEMPLATE_ID")

    sampleData = {
        "name": "John Doe",
    }

    # immediate document generation
    immediate_document = client.generate_document_immediate(template_id=template_id, data=sampleData, render_config=RenderConfig(type="html", target="pdf"))
    print("immediateDocument:", immediate_document)
    
    # document generation
    document = client.generate_document(data=sampleData, render_config=RenderConfig(type="html", target="pdf"), template_id=template_id)
    print("document:", document)

    # start document generation
    job_id = client.start_generate_document(data=sampleData, render_config=RenderConfig(type="html", target="pdf"), template_id=template_id)
    print("startDocument:", job_id)

    # poll for job completion
    job_status = client.poll_for_job_completion(job_id)
    print("jobStatus:", job_status)
    

if __name__ == "__main__":
    main()

    



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CloneTemplateResponse,
    DeleteTemplateResponse,
    GeneratePresignedGetUrlResponse,
    GenerateTemplatePreviewsRequestFormatOpts,
    GenerateTemplatePreviewsRequestFormatOptsFormat,
    GenerateTemplatePreviewsRequestType,
    GenerateTemplatePreviewsResponse,
    GenerateTemplatePreviewsResponsePdfPreview,
    GenerateTemplatePreviewsResponsePngPreview,
    GetTemplateIndexHtmlResponse,
    InitializeTemplateCreationResponse,
    SaveCreatedTemplateRequestPreviewIds,
    SaveCreatedTemplateRequestTemplateInfo,
    SaveCreatedTemplateRequestTemplateInfoCategoriesItem,
    SaveCreatedTemplateRequestTemplateInfoType,
    UpdateTemplateRequestPreviewIds,
    UpdateTemplateRequestTemplateInfo,
    UpdateTemplateRequestTemplateInfoCategoriesItem,
    UpdateTemplateRequestTemplateInfoType,
    UpdateTemplateResponse,
)

__all__ = [
    "CloneTemplateResponse",
    "DeleteTemplateResponse",
    "GeneratePresignedGetUrlResponse",
    "GenerateTemplatePreviewsRequestFormatOpts",
    "GenerateTemplatePreviewsRequestFormatOptsFormat",
    "GenerateTemplatePreviewsRequestType",
    "GenerateTemplatePreviewsResponse",
    "GenerateTemplatePreviewsResponsePdfPreview",
    "GenerateTemplatePreviewsResponsePngPreview",
    "GetTemplateIndexHtmlResponse",
    "InitializeTemplateCreationResponse",
    "SaveCreatedTemplateRequestPreviewIds",
    "SaveCreatedTemplateRequestTemplateInfo",
    "SaveCreatedTemplateRequestTemplateInfoCategoriesItem",
    "SaveCreatedTemplateRequestTemplateInfoType",
    "UpdateTemplateRequestPreviewIds",
    "UpdateTemplateRequestTemplateInfo",
    "UpdateTemplateRequestTemplateInfoCategoriesItem",
    "UpdateTemplateRequestTemplateInfoType",
    "UpdateTemplateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .save_created_template_request_template_info_categories_item import (
    SaveCreatedTemplateRequestTemplateInfoCategoriesItem,
)
from .save_created_template_request_template_info_type import SaveCreatedTemplateRequestTemplateInfoType


class SaveCreatedTemplateRequestTemplateInfo(UniversalBaseModel):
    title: str = pydantic.Field()
    """
    Title of the template
    """

    description: str = pydantic.Field()
    """
    Description of the template
    """

    type: SaveCreatedTemplateRequestTemplateInfoType = pydantic.Field()
    """
    Type of template to be rendered
    """

    sample_data: typing_extensions.Annotated[
        typing.Dict[str, typing.Optional[typing.Any]], FieldMetadata(alias="sampleData")
    ] = pydantic.Field()
    """
    Sample data for the template
    """

    source_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="sourceCode")] = None
    categories: typing.List[SaveCreatedTemplateRequestTemplateInfoCategoriesItem] = pydantic.Field()
    """
    Categories of the template
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.clone_template_response import CloneTemplateResponse
from .types.delete_template_response import DeleteTemplateResponse
from .types.generate_presigned_get_url_response import GeneratePresignedGetUrlResponse
from .types.generate_template_previews_request_format_opts import GenerateTemplatePreviewsRequestFormatOpts
from .types.generate_template_previews_request_type import GenerateTemplatePreviewsRequestType
from .types.generate_template_previews_response import GenerateTemplatePreviewsResponse
from .types.get_template_index_html_response import GetTemplateIndexHtmlResponse
from .types.initialize_template_creation_response import InitializeTemplateCreationResponse
from .types.save_created_template_request_preview_ids import SaveCreatedTemplateRequestPreviewIds
from .types.save_created_template_request_template_info import SaveCreatedTemplateRequestTemplateInfo
from .types.update_template_request_preview_ids import UpdateTemplateRequestPreviewIds
from .types.update_template_request_template_info import UpdateTemplateRequestTemplateInfo
from .types.update_template_response import UpdateTemplateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def initialize_template_creation(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InitializeTemplateCreationResponse]:
        """
        Initializes template creation by generating a unique ID and providing a presigned URL for template ZIP upload. Sets unfinished tag for tracking incomplete templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InitializeTemplateCreationResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "templates/init",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitializeTemplateCreationResponse,
                    parse_obj_as(
                        type_=InitializeTemplateCreationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def save_created_template(
        self,
        template_id: str,
        *,
        template_info: SaveCreatedTemplateRequestTemplateInfo,
        preview_ids: SaveCreatedTemplateRequestPreviewIds,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Finalizes template creation by saving template info to Strapi, copying preview files to permanent storage, and creating template index. Removes unfinished tag upon completion.

        Parameters
        ----------
        template_id : str

        template_info : SaveCreatedTemplateRequestTemplateInfo

        preview_ids : SaveCreatedTemplateRequestPreviewIds

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="POST",
            json={
                "templateInfo": convert_and_respect_annotation_metadata(
                    object_=template_info, annotation=SaveCreatedTemplateRequestTemplateInfo, direction="write"
                ),
                "previewIds": convert_and_respect_annotation_metadata(
                    object_=preview_ids, annotation=SaveCreatedTemplateRequestPreviewIds, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_template(
        self,
        template_id: str,
        *,
        template_info: UpdateTemplateRequestTemplateInfo,
        preview_ids: UpdateTemplateRequestPreviewIds,
        content_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateTemplateResponse]:
        """
        Updates template content, handles S3 storage cleanup for old content, updates template metadata in Strapi, and manages preview files. Removes unfinished tags after successful update.

        Parameters
        ----------
        template_id : str

        template_info : UpdateTemplateRequestTemplateInfo

        preview_ids : UpdateTemplateRequestPreviewIds

        content_id : str
            ID by which the new template content is saved

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateTemplateResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="PUT",
            json={
                "templateInfo": convert_and_respect_annotation_metadata(
                    object_=template_info, annotation=UpdateTemplateRequestTemplateInfo, direction="write"
                ),
                "previewIds": convert_and_respect_annotation_metadata(
                    object_=preview_ids, annotation=UpdateTemplateRequestPreviewIds, direction="write"
                ),
                "contentId": content_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTemplateResponse,
                    parse_obj_as(
                        type_=UpdateTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteTemplateResponse]:
        """
        Deletes a template from Strapi and associated S3 storage. Removes all associated files and metadata.

        Parameters
        ----------
        template_id : str
            ID of the template to be deleted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteTemplateResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteTemplateResponse,
                    parse_obj_as(
                        type_=DeleteTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def extract_template_files(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Extracts contents from an uploaded template ZIP file and stores individual files in the appropriate S3 storage structure.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/unzip",
            method="PATCH",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def generate_template_previews(
        self,
        template_id: str,
        *,
        type: GenerateTemplatePreviewsRequestType,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        format_opts: typing.Optional[GenerateTemplatePreviewsRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GenerateTemplatePreviewsResponse]:
        """
        Creates both PNG and PDF preview files for template visualization. Generates previews in parallel and returns URLs for both formats.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        type : GenerateTemplatePreviewsRequestType
            Type of template to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        format_opts : typing.Optional[GenerateTemplatePreviewsRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GenerateTemplatePreviewsResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/render-previews",
            method="POST",
            json={
                "type": type,
                "data": data,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=GenerateTemplatePreviewsRequestFormatOpts, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GenerateTemplatePreviewsResponse,
                    parse_obj_as(
                        type_=GenerateTemplatePreviewsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def generate_presigned_get_url(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GeneratePresignedGetUrlResponse]:
        """
        Generates a presigned URL for template access. Used for downloading template files from S3 storage.

        Parameters
        ----------
        template_id : str
            ID of the template that is being downloaded

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GeneratePresignedGetUrlResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/presigned-url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GeneratePresignedGetUrlResponse,
                    parse_obj_as(
                        type_=GeneratePresignedGetUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_template_index_html(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTemplateIndexHtmlResponse]:
        """
        Retrieves the template index.html file from S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTemplateIndexHtmlResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/index-html",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTemplateIndexHtmlResponse,
                    parse_obj_as(
                        type_=GetTemplateIndexHtmlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upload_template_index_html(
        self, template_id: str, *, index_html: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Uploads the template index.html file to S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        index_html : str
            New index.html file of the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/index-html",
            method="POST",
            json={
                "indexHtml": index_html,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def clone_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CloneTemplateResponse]:
        """
        Creates a new template by duplicating an existing template's content and metadata. Includes copying preview files and template index.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CloneTemplateResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/clone",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CloneTemplateResponse,
                    parse_obj_as(
                        type_=CloneTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def initialize_template_creation(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InitializeTemplateCreationResponse]:
        """
        Initializes template creation by generating a unique ID and providing a presigned URL for template ZIP upload. Sets unfinished tag for tracking incomplete templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InitializeTemplateCreationResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "templates/init",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitializeTemplateCreationResponse,
                    parse_obj_as(
                        type_=InitializeTemplateCreationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def save_created_template(
        self,
        template_id: str,
        *,
        template_info: SaveCreatedTemplateRequestTemplateInfo,
        preview_ids: SaveCreatedTemplateRequestPreviewIds,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Finalizes template creation by saving template info to Strapi, copying preview files to permanent storage, and creating template index. Removes unfinished tag upon completion.

        Parameters
        ----------
        template_id : str

        template_info : SaveCreatedTemplateRequestTemplateInfo

        preview_ids : SaveCreatedTemplateRequestPreviewIds

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="POST",
            json={
                "templateInfo": convert_and_respect_annotation_metadata(
                    object_=template_info, annotation=SaveCreatedTemplateRequestTemplateInfo, direction="write"
                ),
                "previewIds": convert_and_respect_annotation_metadata(
                    object_=preview_ids, annotation=SaveCreatedTemplateRequestPreviewIds, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_template(
        self,
        template_id: str,
        *,
        template_info: UpdateTemplateRequestTemplateInfo,
        preview_ids: UpdateTemplateRequestPreviewIds,
        content_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateTemplateResponse]:
        """
        Updates template content, handles S3 storage cleanup for old content, updates template metadata in Strapi, and manages preview files. Removes unfinished tags after successful update.

        Parameters
        ----------
        template_id : str

        template_info : UpdateTemplateRequestTemplateInfo

        preview_ids : UpdateTemplateRequestPreviewIds

        content_id : str
            ID by which the new template content is saved

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateTemplateResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="PUT",
            json={
                "templateInfo": convert_and_respect_annotation_metadata(
                    object_=template_info, annotation=UpdateTemplateRequestTemplateInfo, direction="write"
                ),
                "previewIds": convert_and_respect_annotation_metadata(
                    object_=preview_ids, annotation=UpdateTemplateRequestPreviewIds, direction="write"
                ),
                "contentId": content_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTemplateResponse,
                    parse_obj_as(
                        type_=UpdateTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteTemplateResponse]:
        """
        Deletes a template from Strapi and associated S3 storage. Removes all associated files and metadata.

        Parameters
        ----------
        template_id : str
            ID of the template to be deleted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteTemplateResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteTemplateResponse,
                    parse_obj_as(
                        type_=DeleteTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def extract_template_files(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Extracts contents from an uploaded template ZIP file and stores individual files in the appropriate S3 storage structure.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/unzip",
            method="PATCH",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def generate_template_previews(
        self,
        template_id: str,
        *,
        type: GenerateTemplatePreviewsRequestType,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        format_opts: typing.Optional[GenerateTemplatePreviewsRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GenerateTemplatePreviewsResponse]:
        """
        Creates both PNG and PDF preview files for template visualization. Generates previews in parallel and returns URLs for both formats.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        type : GenerateTemplatePreviewsRequestType
            Type of template to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        format_opts : typing.Optional[GenerateTemplatePreviewsRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GenerateTemplatePreviewsResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/render-previews",
            method="POST",
            json={
                "type": type,
                "data": data,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=GenerateTemplatePreviewsRequestFormatOpts, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GenerateTemplatePreviewsResponse,
                    parse_obj_as(
                        type_=GenerateTemplatePreviewsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def generate_presigned_get_url(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GeneratePresignedGetUrlResponse]:
        """
        Generates a presigned URL for template access. Used for downloading template files from S3 storage.

        Parameters
        ----------
        template_id : str
            ID of the template that is being downloaded

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GeneratePresignedGetUrlResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/presigned-url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GeneratePresignedGetUrlResponse,
                    parse_obj_as(
                        type_=GeneratePresignedGetUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_template_index_html(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTemplateIndexHtmlResponse]:
        """
        Retrieves the template index.html file from S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTemplateIndexHtmlResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/index-html",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTemplateIndexHtmlResponse,
                    parse_obj_as(
                        type_=GetTemplateIndexHtmlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upload_template_index_html(
        self, template_id: str, *, index_html: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Uploads the template index.html file to S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        index_html : str
            New index.html file of the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/index-html",
            method="POST",
            json={
                "indexHtml": index_html,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def clone_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CloneTemplateResponse]:
        """
        Creates a new template by duplicating an existing template's content and metadata. Includes copying preview files and template index.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CloneTemplateResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"templates/{jsonable_encoder(template_id)}/clone",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CloneTemplateResponse,
                    parse_obj_as(
                        type_=CloneTemplateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .get_job_status_response import GetJobStatusResponse
from .get_job_status_response_format_opts import GetJobStatusResponseFormatOpts
from .get_job_status_response_format_opts_format import GetJobStatusResponseFormatOptsFormat
from .get_job_status_response_output import GetJobStatusResponseOutput
from .get_job_status_response_output_data import GetJobStatusResponseOutputData
from .get_job_status_response_output_metadata import GetJobStatusResponseOutputMetadata
from .initialize_render_job_request_format_opts import InitializeRenderJobRequestFormatOpts
from .initialize_render_job_request_format_opts_format import InitializeRenderJobRequestFormatOptsFormat
from .initialize_render_job_request_target import InitializeRenderJobRequestTarget
from .initialize_render_job_request_type import InitializeRenderJobRequestType
from .initialize_render_job_response import InitializeRenderJobResponse
from .initialize_render_job_response_target import InitializeRenderJobResponseTarget
from .start_immediate_render_request_format_opts import StartImmediateRenderRequestFormatOpts
from .start_immediate_render_request_format_opts_format import StartImmediateRenderRequestFormatOptsFormat
from .start_immediate_render_request_target import StartImmediateRenderRequestTarget
from .start_immediate_render_request_type import StartImmediateRenderRequestType
from .start_immediate_render_response import StartImmediateRenderResponse
from .start_render_job_response import StartRenderJobResponse
from .start_render_job_response_format_opts import StartRenderJobResponseFormatOpts
from .start_render_job_response_format_opts_format import StartRenderJobResponseFormatOptsFormat
from .start_render_job_response_output import StartRenderJobResponseOutput
from .start_render_job_response_output_data import StartRenderJobResponseOutputData
from .start_render_job_response_output_metadata import StartRenderJobResponseOutputMetadata
from .start_render_job_response_target import StartRenderJobResponseTarget

__all__ = [
    "GetJobStatusResponse",
    "GetJobStatusResponseFormatOpts",
    "GetJobStatusResponseFormatOptsFormat",
    "GetJobStatusResponseOutput",
    "GetJobStatusResponseOutputData",
    "GetJobStatusResponseOutputMetadata",
    "InitializeRenderJobRequestFormatOpts",
    "InitializeRenderJobRequestFormatOptsFormat",
    "InitializeRenderJobRequestTarget",
    "InitializeRenderJobRequestType",
    "InitializeRenderJobResponse",
    "InitializeRenderJobResponseTarget",
    "StartImmediateRenderRequestFormatOpts",
    "StartImmediateRenderRequestFormatOptsFormat",
    "StartImmediateRenderRequestTarget",
    "StartImmediateRenderRequestType",
    "StartImmediateRenderResponse",
    "StartRenderJobResponse",
    "StartRenderJobResponseFormatOpts",
    "StartRenderJobResponseFormatOptsFormat",
    "StartRenderJobResponseOutput",
    "StartRenderJobResponseOutputData",
    "StartRenderJobResponseOutputMetadata",
    "StartRenderJobResponseTarget",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawDocumentsClient, RawDocumentsClient
from .types.get_job_status_response import GetJobStatusResponse
from .types.initialize_render_job_request_format_opts import InitializeRenderJobRequestFormatOpts
from .types.initialize_render_job_request_target import InitializeRenderJobRequestTarget
from .types.initialize_render_job_request_type import InitializeRenderJobRequestType
from .types.initialize_render_job_response import InitializeRenderJobResponse
from .types.start_immediate_render_request_format_opts import StartImmediateRenderRequestFormatOpts
from .types.start_immediate_render_request_target import StartImmediateRenderRequestTarget
from .types.start_immediate_render_request_type import StartImmediateRenderRequestType
from .types.start_immediate_render_response import StartImmediateRenderResponse
from .types.start_render_job_response import StartRenderJobResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDocumentsClient
        """
        return self._raw_client

    def initialize_render_job(
        self,
        *,
        type: InitializeRenderJobRequestType,
        target: InitializeRenderJobRequestTarget,
        data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[InitializeRenderJobRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InitializeRenderJobResponse:
        """
        Creates a new render job with a unique ID, sets up S3 storage for template and data files, and generates presigned upload URLs if needed. Requires subscription check.

        Parameters
        ----------
        type : InitializeRenderJobRequestType
            Type of template to be rendered

        target : InitializeRenderJobRequestTarget
            Type of output to be rendered

        data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[InitializeRenderJobRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitializeRenderJobResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.documents.initialize_render_job(
            type="docx",
            target="pdf",
        )
        """
        _response = self._raw_client.initialize_render_job(
            type=type,
            target=target,
            data=data,
            template_id=template_id,
            format_opts=format_opts,
            request_options=request_options,
        )
        return _response.data

    def start_render_job(
        self,
        job_id: str,
        *,
        should_wait_for_render_completion: typing.Optional[bool] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartRenderJobResponse:
        """
        Takes a previously initialized job, updates its status to in-progress, and triggers the rendering process using Puppeteer. Can optionally wait for render completion.

        Parameters
        ----------
        job_id : str

        should_wait_for_render_completion : typing.Optional[bool]
            Whether to wait for the render job to complete, if false, the job will be returned immediately

        upload_presigned_s_3_url : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartRenderJobResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.documents.start_render_job(
            job_id="jobId",
        )
        """
        _response = self._raw_client.start_render_job(
            job_id,
            should_wait_for_render_completion=should_wait_for_render_completion,
            upload_presigned_s_3_url=upload_presigned_s_3_url,
            request_options=request_options,
        )
        return _response.data

    def start_immediate_render(
        self,
        *,
        type: StartImmediateRenderRequestType,
        target: StartImmediateRenderRequestTarget,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[StartImmediateRenderRequestFormatOpts] = OMIT,
        template: typing.Optional[str] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartImmediateRenderResponse:
        """
        Combines initialization and rendering in one step. Creates a job, uploads template/data directly, starts rendering, and adds the document to Strapi. Requires subscription check.

        Parameters
        ----------
        type : StartImmediateRenderRequestType
            Type of template to be rendered

        target : StartImmediateRenderRequestTarget
            Type of output to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[StartImmediateRenderRequestFormatOpts]
            Format options for the rendered document

        template : typing.Optional[str]
            index.html or ejs file of the template as a string

        upload_presigned_s_3_url : typing.Optional[str]
            Presigned URL to upload the data for the render job to S3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartImmediateRenderResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.documents.start_immediate_render(
            type="docx",
            target="pdf",
            data={"key": "value"},
        )
        """
        _response = self._raw_client.start_immediate_render(
            type=type,
            target=target,
            data=data,
            template_id=template_id,
            format_opts=format_opts,
            template=template,
            upload_presigned_s_3_url=upload_presigned_s_3_url,
            request_options=request_options,
        )
        return _response.data

    def get_job_status(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetJobStatusResponse:
        """
        Fetches detailed job information from S3 storage including job status, template ID, target format, and output details if available.

        Parameters
        ----------
        job_id : str
            ID of the render job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetJobStatusResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.documents.get_job_status(
            job_id="jobId",
        )
        """
        _response = self._raw_client.get_job_status(job_id, request_options=request_options)
        return _response.data


class AsyncDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDocumentsClient
        """
        return self._raw_client

    async def initialize_render_job(
        self,
        *,
        type: InitializeRenderJobRequestType,
        target: InitializeRenderJobRequestTarget,
        data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[InitializeRenderJobRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InitializeRenderJobResponse:
        """
        Creates a new render job with a unique ID, sets up S3 storage for template and data files, and generates presigned upload URLs if needed. Requires subscription check.

        Parameters
        ----------
        type : InitializeRenderJobRequestType
            Type of template to be rendered

        target : InitializeRenderJobRequestTarget
            Type of output to be rendered

        data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[InitializeRenderJobRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitializeRenderJobResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.initialize_render_job(
                type="docx",
                target="pdf",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.initialize_render_job(
            type=type,
            target=target,
            data=data,
            template_id=template_id,
            format_opts=format_opts,
            request_options=request_options,
        )
        return _response.data

    async def start_render_job(
        self,
        job_id: str,
        *,
        should_wait_for_render_completion: typing.Optional[bool] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartRenderJobResponse:
        """
        Takes a previously initialized job, updates its status to in-progress, and triggers the rendering process using Puppeteer. Can optionally wait for render completion.

        Parameters
        ----------
        job_id : str

        should_wait_for_render_completion : typing.Optional[bool]
            Whether to wait for the render job to complete, if false, the job will be returned immediately

        upload_presigned_s_3_url : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartRenderJobResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.start_render_job(
                job_id="jobId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.start_render_job(
            job_id,
            should_wait_for_render_completion=should_wait_for_render_completion,
            upload_presigned_s_3_url=upload_presigned_s_3_url,
            request_options=request_options,
        )
        return _response.data

    async def start_immediate_render(
        self,
        *,
        type: StartImmediateRenderRequestType,
        target: StartImmediateRenderRequestTarget,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[StartImmediateRenderRequestFormatOpts] = OMIT,
        template: typing.Optional[str] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartImmediateRenderResponse:
        """
        Combines initialization and rendering in one step. Creates a job, uploads template/data directly, starts rendering, and adds the document to Strapi. Requires subscription check.

        Parameters
        ----------
        type : StartImmediateRenderRequestType
            Type of template to be rendered

        target : StartImmediateRenderRequestTarget
            Type of output to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[StartImmediateRenderRequestFormatOpts]
            Format options for the rendered document

        template : typing.Optional[str]
            index.html or ejs file of the template as a string

        upload_presigned_s_3_url : typing.Optional[str]
            Presigned URL to upload the data for the render job to S3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartImmediateRenderResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.start_immediate_render(
                type="docx",
                target="pdf",
                data={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.start_immediate_render(
            type=type,
            target=target,
            data=data,
            template_id=template_id,
            format_opts=format_opts,
            template=template,
            upload_presigned_s_3_url=upload_presigned_s_3_url,
            request_options=request_options,
        )
        return _response.data

    async def get_job_status(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetJobStatusResponse:
        """
        Fetches detailed job information from S3 storage including job status, template ID, target format, and output details if available.

        Parameters
        ----------
        job_id : str
            ID of the render job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetJobStatusResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.get_job_status(
                job_id="jobId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_job_status(job_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .clone_template_response import CloneTemplateResponse
from .delete_template_response import DeleteTemplateResponse
from .generate_presigned_get_url_response import GeneratePresignedGetUrlResponse
from .generate_template_previews_request_format_opts import GenerateTemplatePreviewsRequestFormatOpts
from .generate_template_previews_request_format_opts_format import GenerateTemplatePreviewsRequestFormatOptsFormat
from .generate_template_previews_request_type import GenerateTemplatePreviewsRequestType
from .generate_template_previews_response import GenerateTemplatePreviewsResponse
from .generate_template_previews_response_pdf_preview import GenerateTemplatePreviewsResponsePdfPreview
from .generate_template_previews_response_png_preview import GenerateTemplatePreviewsResponsePngPreview
from .get_template_index_html_response import GetTemplateIndexHtmlResponse
from .initialize_template_creation_response import InitializeTemplateCreationResponse
from .save_created_template_request_preview_ids import SaveCreatedTemplateRequestPreviewIds
from .save_created_template_request_template_info import SaveCreatedTemplateRequestTemplateInfo
from .save_created_template_request_template_info_categories_item import (
    SaveCreatedTemplateRequestTemplateInfoCategoriesItem,
)
from .save_created_template_request_template_info_type import SaveCreatedTemplateRequestTemplateInfoType
from .update_template_request_preview_ids import UpdateTemplateRequestPreviewIds
from .update_template_request_template_info import UpdateTemplateRequestTemplateInfo
from .update_template_request_template_info_categories_item import UpdateTemplateRequestTemplateInfoCategoriesItem
from .update_template_request_template_info_type import UpdateTemplateRequestTemplateInfoType
from .update_template_response import UpdateTemplateResponse

__all__ = [
    "CloneTemplateResponse",
    "DeleteTemplateResponse",
    "GeneratePresignedGetUrlResponse",
    "GenerateTemplatePreviewsRequestFormatOpts",
    "GenerateTemplatePreviewsRequestFormatOptsFormat",
    "GenerateTemplatePreviewsRequestType",
    "GenerateTemplatePreviewsResponse",
    "GenerateTemplatePreviewsResponsePdfPreview",
    "GenerateTemplatePreviewsResponsePngPreview",
    "GetTemplateIndexHtmlResponse",
    "InitializeTemplateCreationResponse",
    "SaveCreatedTemplateRequestPreviewIds",
    "SaveCreatedTemplateRequestTemplateInfo",
    "SaveCreatedTemplateRequestTemplateInfoCategoriesItem",
    "SaveCreatedTemplateRequestTemplateInfoType",
    "UpdateTemplateRequestPreviewIds",
    "UpdateTemplateRequestTemplateInfo",
    "UpdateTemplateRequestTemplateInfoCategoriesItem",
    "UpdateTemplateRequestTemplateInfoType",
    "UpdateTemplateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .start_render_job_response_format_opts import StartRenderJobResponseFormatOpts
from .start_render_job_response_output import StartRenderJobResponseOutput
from .start_render_job_response_target import StartRenderJobResponseTarget


class StartRenderJobResponse(UniversalBaseModel):
    job_id: typing_extensions.Annotated[str, FieldMetadata(alias="jobId")] = pydantic.Field()
    """
    ID of the render job
    """

    template_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="templateId")] = pydantic.Field(
        default=None
    )
    """
    ID of the template being used
    """

    target: typing.Optional[StartRenderJobResponseTarget] = pydantic.Field(default=None)
    """
    Type of output to be rendered
    """

    upload_presigned_s_3_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="uploadPresignedS3Url")
    ] = pydantic.Field(default=None)
    """
    Presigned URL to upload the rendered output to S3
    """

    format_opts: typing_extensions.Annotated[
        typing.Optional[StartRenderJobResponseFormatOpts], FieldMetadata(alias="formatOpts")
    ] = pydantic.Field(default=None)
    """
    Format options for the rendered document
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status of the render job
    """

    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the render job was successful
    """

    output: typing.Optional[StartRenderJobResponseOutput] = None
    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error that occurred during render
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

StartRenderJobResponseFormatOptsFormat = typing.Union[
    typing.Literal["letter", "legal", "tabloid", "ledger", "a0", "a1", "a2", "a3", "a4", "a5", "a6"], typing.Any
]



import os
import typing
import json
import io
import time
from pogodoc.client.client import PogodocApi
from pogodoc.utils import RenderConfig, upload_to_s3_with_url
from pogodoc.client.templates.types import SaveCreatedTemplateRequestPreviewIds, SaveCreatedTemplateRequestTemplateInfo, UpdateTemplateRequestPreviewIds, UpdateTemplateRequestTemplateInfo

class PogodocClient(PogodocApi):
    def __init__(self, token: str = None, base_url: str = None):
        """Initializes a new instance of the PogodocClient."""
        token = token or os.getenv("POGODOC_API_TOKEN")
        base_url = base_url or os.getenv("POGODOC_BASE_URL")

        if not token:
            raise ValueError("API token is required. Please provide it either as a parameter or set the API_TOKEN environment variable.")

        super().__init__(token=token, base_url=base_url)
   
    def save_template(self, path: str, template_info:SaveCreatedTemplateRequestTemplateInfo):
        """
        Saves a new template from a local file path.
        This method reads a template from a .zip file, uploads it, and saves it in Pogodoc.
        It is a convenient wrapper around `save_template_from_file_stream`.
        """
        zip = open(path, "rb")
        zip_length = os.path.getsize(path)
        return self.save_template_from_file_stream(payload=zip, payload_length=zip_length, template_info=template_info)
    
    def save_template_from_file_stream(self, payload:io.BufferedReader, payload_length:int, template_info:SaveCreatedTemplateRequestTemplateInfo):
        """
        Saves a new template from a file stream.
        This is the core method for creating templates. It uploads a template from a stream,
        generates previews, and saves it with the provided metadata.
        """
        init_response = self.templates.initialize_template_creation()

        template_id = init_response.template_id

        upload_to_s3_with_url(presigned_url=init_response.presigned_template_upload_url, payload=payload, payload_length=payload_length, content_type="application/zip")

        self.templates.extract_template_files(template_id)

        preview_response = self.templates.generate_template_previews(template_id,
            type=template_info.type,
            data=template_info.sample_data
        )

        self.templates.save_created_template(template_id, 
            template_info=template_info,
            preview_ids=SaveCreatedTemplateRequestPreviewIds(
                png_job_id=preview_response.png_preview.job_id,
                pdf_job_id=preview_response.pdf_preview.job_id
            )
        )

        return template_id
    
    def update_template(self, template_id: str, path: str, template_info:UpdateTemplateRequestTemplateInfo):
        """
        Updates an existing template from a local file path.
        This method reads a new version of a template from a .zip file, uploads it,
        and updates the existing template in Pogodoc.
        It is a convenient wrapper around `update_template_from_file_stream`.
        """
        if not os.path.exists(path):
            raise FileNotFoundError(f"Template file not found at path: {path}")

        zip = open(path, "rb")
        zip_length = os.path.getsize(path)

        return self.update_template_from_file_stream(
            template_id=template_id,
            payload=zip,
            payload_length=zip_length,
            template_info=template_info
        )

    def update_template_from_file_stream(self, template_id: str, payload:io.BufferedReader, payload_length:int, template_info:UpdateTemplateRequestTemplateInfo):
        """
        Updates an existing template from a file stream.
        This is the core method for updating templates. It uploads a new template version from a stream,
        generates new previews, and updates the template with the provided metadata.
        """
        init_response = self.templates.initialize_template_creation()
        content_id = init_response.template_id

        upload_to_s3_with_url(
            presigned_url=init_response.presigned_template_upload_url,
            payload=payload,
            payload_length=payload_length,
            content_type="application/zip"
        )

        self.templates.extract_template_files(content_id)

        preview_response = self.templates.generate_template_previews(
            template_id=content_id,
            type=template_info.type,
            data=template_info.sample_data
        )

        updated_template_response = self.templates.update_template(
            template_id=template_id,
            content_id=content_id,
            template_info=template_info,
            preview_ids= UpdateTemplateRequestPreviewIds(
                png_job_id=preview_response.png_preview.job_id,
                pdf_job_id=preview_response.pdf_preview.job_id
            )
        )
        return updated_template_response

    def generate_document(self, data: dict, render_config: RenderConfig, personal_upload_presigned_s3_url:typing.Optional[str] = None,  template: typing.Optional[str] = None, template_id: typing.Optional[str] = None):
        """
        Generates a document by starting a job and polling for its completion.
        This is the recommended method for most use cases, especially for larger documents or when you want a simple fire-and-forget operation.
        It first calls `start_generate_document` to begin the process, then `poll_for_job_completion` to wait for the result.
        You must provide either a `template_id` of a saved template or a `template` string.
        """
        job_id = self.start_generate_document(data, render_config, personal_upload_presigned_s3_url, template, template_id)
        return self.poll_for_job_completion(job_id)

    def start_generate_document(self, data: dict, render_config: RenderConfig, personal_upload_presigned_s3_url:typing.Optional[str] = None,  template: typing.Optional[str] = None, template_id: typing.Optional[str] = None):
        """
        Starts an asynchronous document generation job.
        This is a lower-level method that only initializes the job.
        You can use this if you want to implement your own polling logic.
        It returns the initial job status, which includes the `job_id`.
        Use `poll_for_job_completion` with the `job_id` to get the final result.
        You must provide either a `template_id` of a saved template or a `template` string.
        """

        render_options = {
            "type": render_config.type,
            "target": render_config.target,
        }
        if render_config.format_opts:
            render_options["format_opts"] = render_config.format_opts

        init_response = self.documents.initialize_render_job(
            data=data,
            template_id=template_id,
            **render_options
        )
        
        if data and init_response.presigned_data_upload_url:
            data_string = json.dumps(data)
            data_stream = io.BytesIO(data_string.encode('utf-8'))
            data_length = len(data_string)

            upload_to_s3_with_url(
                presigned_url=init_response.presigned_data_upload_url,
                payload=data_stream,
                payload_length=data_length,
                content_type="application/json"
            )

        if template and init_response.presigned_template_upload_url:
            upload_to_s3_with_url(
                presigned_url=init_response.presigned_template_upload_url,
                payload=io.BytesIO(template.encode('utf-8')),
                payload_length=len(template),
                content_type="text/html"
            )

        response = self.documents.start_render_job(
            job_id=init_response.job_id,
            upload_presigned_s_3_url=personal_upload_presigned_s3_url
        )

        return response.job_id
    
    
    def generate_document_immediate(self, data: dict, render_config: RenderConfig, template: typing.Optional[str] = None, template_id: typing.Optional[str] = None):
        """
        Generates a document and returns the result immediately.
        Use this method for quick, synchronous rendering of small documents.
        The result is returned directly in the response.
        For larger documents or when you need to handle rendering asynchronously, use `generate_document`.
        You must provide either a `template_id` of a saved template or a `template` string.
        """
        return self.documents.start_immediate_render(
            template_id=template_id,
            template=template,
            type=render_config.type,
            target=render_config.target,
            format_opts=render_config.format_opts,
            data=data,
        )
    
    def poll_for_job_completion(self, job_id: str, max_attempts: int = 60, interval_ms: int = 500):
        """
        Polls for the completion of a rendering job.
        This method repeatedly checks the status of a job until it is 'done'.
        """
        time.sleep(1)
        for attempt in range(max_attempts):
            job = self.documents.get_job_status(job_id)
            if job.status == "done":
                return job
            time.sleep(interval_ms / 1000)

        return self.documents.get_job_status(job_id)


# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from . import documents, templates
from .client import AsyncPogodocApi, PogodocApi
from .documents import (
    GetJobStatusResponse,
    GetJobStatusResponseFormatOpts,
    GetJobStatusResponseFormatOptsFormat,
    GetJobStatusResponseOutput,
    GetJobStatusResponseOutputData,
    GetJobStatusResponseOutputMetadata,
    InitializeRenderJobRequestFormatOpts,
    InitializeRenderJobRequestFormatOptsFormat,
    InitializeRenderJobRequestTarget,
    InitializeRenderJobRequestType,
    InitializeRenderJobResponse,
    InitializeRenderJobResponseTarget,
    StartImmediateRenderRequestFormatOpts,
    StartImmediateRenderRequestFormatOptsFormat,
    StartImmediateRenderRequestTarget,
    StartImmediateRenderRequestType,
    StartImmediateRenderResponse,
    StartRenderJobResponse,
    StartRenderJobResponseFormatOpts,
    StartRenderJobResponseFormatOptsFormat,
    StartRenderJobResponseOutput,
    StartRenderJobResponseOutputData,
    StartRenderJobResponseOutputMetadata,
    StartRenderJobResponseTarget,
)
from .environment import PogodocApiEnvironment
from .templates import (
    CloneTemplateResponse,
    DeleteTemplateResponse,
    GeneratePresignedGetUrlResponse,
    GenerateTemplatePreviewsRequestFormatOpts,
    GenerateTemplatePreviewsRequestFormatOptsFormat,
    GenerateTemplatePreviewsRequestType,
    GenerateTemplatePreviewsResponse,
    GenerateTemplatePreviewsResponsePdfPreview,
    GenerateTemplatePreviewsResponsePngPreview,
    GetTemplateIndexHtmlResponse,
    InitializeTemplateCreationResponse,
    SaveCreatedTemplateRequestPreviewIds,
    SaveCreatedTemplateRequestTemplateInfo,
    SaveCreatedTemplateRequestTemplateInfoCategoriesItem,
    SaveCreatedTemplateRequestTemplateInfoType,
    UpdateTemplateRequestPreviewIds,
    UpdateTemplateRequestTemplateInfo,
    UpdateTemplateRequestTemplateInfoCategoriesItem,
    UpdateTemplateRequestTemplateInfoType,
    UpdateTemplateResponse,
)

__all__ = [
    "AsyncPogodocApi",
    "CloneTemplateResponse",
    "DeleteTemplateResponse",
    "GeneratePresignedGetUrlResponse",
    "GenerateTemplatePreviewsRequestFormatOpts",
    "GenerateTemplatePreviewsRequestFormatOptsFormat",
    "GenerateTemplatePreviewsRequestType",
    "GenerateTemplatePreviewsResponse",
    "GenerateTemplatePreviewsResponsePdfPreview",
    "GenerateTemplatePreviewsResponsePngPreview",
    "GetJobStatusResponse",
    "GetJobStatusResponseFormatOpts",
    "GetJobStatusResponseFormatOptsFormat",
    "GetJobStatusResponseOutput",
    "GetJobStatusResponseOutputData",
    "GetJobStatusResponseOutputMetadata",
    "GetTemplateIndexHtmlResponse",
    "InitializeRenderJobRequestFormatOpts",
    "InitializeRenderJobRequestFormatOptsFormat",
    "InitializeRenderJobRequestTarget",
    "InitializeRenderJobRequestType",
    "InitializeRenderJobResponse",
    "InitializeRenderJobResponseTarget",
    "InitializeTemplateCreationResponse",
    "PogodocApi",
    "PogodocApiEnvironment",
    "SaveCreatedTemplateRequestPreviewIds",
    "SaveCreatedTemplateRequestTemplateInfo",
    "SaveCreatedTemplateRequestTemplateInfoCategoriesItem",
    "SaveCreatedTemplateRequestTemplateInfoType",
    "StartImmediateRenderRequestFormatOpts",
    "StartImmediateRenderRequestFormatOptsFormat",
    "StartImmediateRenderRequestTarget",
    "StartImmediateRenderRequestType",
    "StartImmediateRenderResponse",
    "StartRenderJobResponse",
    "StartRenderJobResponseFormatOpts",
    "StartRenderJobResponseFormatOptsFormat",
    "StartRenderJobResponseOutput",
    "StartRenderJobResponseOutputData",
    "StartRenderJobResponseOutputMetadata",
    "StartRenderJobResponseTarget",
    "UpdateTemplateRequestPreviewIds",
    "UpdateTemplateRequestTemplateInfo",
    "UpdateTemplateRequestTemplateInfoCategoriesItem",
    "UpdateTemplateRequestTemplateInfoType",
    "UpdateTemplateResponse",
    "documents",
    "templates",
]



__all__ = [
    "PogodocClient",
    "RenderConfig",
    "CloneTemplateResponse",
    "DeleteTemplateResponse",
    "GeneratePresignedGetUrlResponse",
    "GenerateTemplatePreviewsRequestFormatOpts",
    "GenerateTemplatePreviewsRequestFormatOptsFormat",
    "GenerateTemplatePreviewsRequestType",
    "GenerateTemplatePreviewsResponse",
    "GenerateTemplatePreviewsResponsePdfPreview",
    "GenerateTemplatePreviewsResponsePngPreview",
    "GetTemplateIndexHtmlResponse",
    "InitializeTemplateCreationResponse",
    "SaveCreatedTemplateRequestPreviewIds",
    "SaveCreatedTemplateRequestTemplateInfo",
    "SaveCreatedTemplateRequestTemplateInfoCategoriesItem",
    "SaveCreatedTemplateRequestTemplateInfoType",
    "UpdateTemplateRequestPreviewIds",
    "UpdateTemplateRequestTemplateInfo",
    "UpdateTemplateRequestTemplateInfoCategoriesItem",
    "UpdateTemplateRequestTemplateInfoType",
    "UpdateTemplateResponse",
    "GetJobStatusResponse",
    "GetJobStatusResponseOutput",
    "GetJobStatusResponseOutputData",
    "GetJobStatusResponseOutputMetadata",
    "GetJobStatusResponseTarget",
    "GetJobStatusResponseType",
    "InitializeRenderJobRequestFormatOpts",
    "InitializeRenderJobRequestFormatOptsFormat",
    "InitializeRenderJobRequestTarget",
    "InitializeRenderJobRequestType",
    "InitializeRenderJobResponse",
    "InitializeRenderJobResponseTarget",
    "StartImmediateRenderRequestFormatOpts",
    "StartImmediateRenderRequestFormatOptsFormat",
    "StartImmediateRenderRequestTarget",
    "StartImmediateRenderRequestType",
    "StartImmediateRenderResponse",
    "StartRenderJobResponse",
]
from .sdk import PogodocClient
from .utils import RenderConfig
from .client.templates.types import *
from .client.documents.types import *

__version__ = "0.2.1"


# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.get_job_status_response import GetJobStatusResponse
from .types.initialize_render_job_request_format_opts import InitializeRenderJobRequestFormatOpts
from .types.initialize_render_job_request_target import InitializeRenderJobRequestTarget
from .types.initialize_render_job_request_type import InitializeRenderJobRequestType
from .types.initialize_render_job_response import InitializeRenderJobResponse
from .types.start_immediate_render_request_format_opts import StartImmediateRenderRequestFormatOpts
from .types.start_immediate_render_request_target import StartImmediateRenderRequestTarget
from .types.start_immediate_render_request_type import StartImmediateRenderRequestType
from .types.start_immediate_render_response import StartImmediateRenderResponse
from .types.start_render_job_response import StartRenderJobResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def initialize_render_job(
        self,
        *,
        type: InitializeRenderJobRequestType,
        target: InitializeRenderJobRequestTarget,
        data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[InitializeRenderJobRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[InitializeRenderJobResponse]:
        """
        Creates a new render job with a unique ID, sets up S3 storage for template and data files, and generates presigned upload URLs if needed. Requires subscription check.

        Parameters
        ----------
        type : InitializeRenderJobRequestType
            Type of template to be rendered

        target : InitializeRenderJobRequestTarget
            Type of output to be rendered

        data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[InitializeRenderJobRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InitializeRenderJobResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "documents/init",
            method="POST",
            json={
                "data": data,
                "type": type,
                "target": target,
                "templateId": template_id,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=InitializeRenderJobRequestFormatOpts, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitializeRenderJobResponse,
                    parse_obj_as(
                        type_=InitializeRenderJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def start_render_job(
        self,
        job_id: str,
        *,
        should_wait_for_render_completion: typing.Optional[bool] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[StartRenderJobResponse]:
        """
        Takes a previously initialized job, updates its status to in-progress, and triggers the rendering process using Puppeteer. Can optionally wait for render completion.

        Parameters
        ----------
        job_id : str

        should_wait_for_render_completion : typing.Optional[bool]
            Whether to wait for the render job to complete, if false, the job will be returned immediately

        upload_presigned_s_3_url : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[StartRenderJobResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"documents/{jsonable_encoder(job_id)}/render",
            method="POST",
            json={
                "shouldWaitForRenderCompletion": should_wait_for_render_completion,
                "uploadPresignedS3Url": upload_presigned_s_3_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartRenderJobResponse,
                    parse_obj_as(
                        type_=StartRenderJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def start_immediate_render(
        self,
        *,
        type: StartImmediateRenderRequestType,
        target: StartImmediateRenderRequestTarget,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[StartImmediateRenderRequestFormatOpts] = OMIT,
        template: typing.Optional[str] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[StartImmediateRenderResponse]:
        """
        Combines initialization and rendering in one step. Creates a job, uploads template/data directly, starts rendering, and adds the document to Strapi. Requires subscription check.

        Parameters
        ----------
        type : StartImmediateRenderRequestType
            Type of template to be rendered

        target : StartImmediateRenderRequestTarget
            Type of output to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[StartImmediateRenderRequestFormatOpts]
            Format options for the rendered document

        template : typing.Optional[str]
            index.html or ejs file of the template as a string

        upload_presigned_s_3_url : typing.Optional[str]
            Presigned URL to upload the data for the render job to S3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[StartImmediateRenderResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "documents/immediate-render",
            method="POST",
            json={
                "type": type,
                "target": target,
                "templateId": template_id,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=StartImmediateRenderRequestFormatOpts, direction="write"
                ),
                "data": data,
                "template": template,
                "uploadPresignedS3Url": upload_presigned_s_3_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartImmediateRenderResponse,
                    parse_obj_as(
                        type_=StartImmediateRenderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_job_status(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetJobStatusResponse]:
        """
        Fetches detailed job information from S3 storage including job status, template ID, target format, and output details if available.

        Parameters
        ----------
        job_id : str
            ID of the render job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetJobStatusResponse]
            Default Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetJobStatusResponse,
                    parse_obj_as(
                        type_=GetJobStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def initialize_render_job(
        self,
        *,
        type: InitializeRenderJobRequestType,
        target: InitializeRenderJobRequestTarget,
        data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[InitializeRenderJobRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[InitializeRenderJobResponse]:
        """
        Creates a new render job with a unique ID, sets up S3 storage for template and data files, and generates presigned upload URLs if needed. Requires subscription check.

        Parameters
        ----------
        type : InitializeRenderJobRequestType
            Type of template to be rendered

        target : InitializeRenderJobRequestTarget
            Type of output to be rendered

        data : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[InitializeRenderJobRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InitializeRenderJobResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "documents/init",
            method="POST",
            json={
                "data": data,
                "type": type,
                "target": target,
                "templateId": template_id,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=InitializeRenderJobRequestFormatOpts, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InitializeRenderJobResponse,
                    parse_obj_as(
                        type_=InitializeRenderJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def start_render_job(
        self,
        job_id: str,
        *,
        should_wait_for_render_completion: typing.Optional[bool] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[StartRenderJobResponse]:
        """
        Takes a previously initialized job, updates its status to in-progress, and triggers the rendering process using Puppeteer. Can optionally wait for render completion.

        Parameters
        ----------
        job_id : str

        should_wait_for_render_completion : typing.Optional[bool]
            Whether to wait for the render job to complete, if false, the job will be returned immediately

        upload_presigned_s_3_url : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[StartRenderJobResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"documents/{jsonable_encoder(job_id)}/render",
            method="POST",
            json={
                "shouldWaitForRenderCompletion": should_wait_for_render_completion,
                "uploadPresignedS3Url": upload_presigned_s_3_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartRenderJobResponse,
                    parse_obj_as(
                        type_=StartRenderJobResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def start_immediate_render(
        self,
        *,
        type: StartImmediateRenderRequestType,
        target: StartImmediateRenderRequestTarget,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        template_id: typing.Optional[str] = OMIT,
        format_opts: typing.Optional[StartImmediateRenderRequestFormatOpts] = OMIT,
        template: typing.Optional[str] = OMIT,
        upload_presigned_s_3_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[StartImmediateRenderResponse]:
        """
        Combines initialization and rendering in one step. Creates a job, uploads template/data directly, starts rendering, and adds the document to Strapi. Requires subscription check.

        Parameters
        ----------
        type : StartImmediateRenderRequestType
            Type of template to be rendered

        target : StartImmediateRenderRequestTarget
            Type of output to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        template_id : typing.Optional[str]
            ID of the template to be used

        format_opts : typing.Optional[StartImmediateRenderRequestFormatOpts]
            Format options for the rendered document

        template : typing.Optional[str]
            index.html or ejs file of the template as a string

        upload_presigned_s_3_url : typing.Optional[str]
            Presigned URL to upload the data for the render job to S3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[StartImmediateRenderResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "documents/immediate-render",
            method="POST",
            json={
                "type": type,
                "target": target,
                "templateId": template_id,
                "formatOpts": convert_and_respect_annotation_metadata(
                    object_=format_opts, annotation=StartImmediateRenderRequestFormatOpts, direction="write"
                ),
                "data": data,
                "template": template,
                "uploadPresignedS3Url": upload_presigned_s_3_url,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartImmediateRenderResponse,
                    parse_obj_as(
                        type_=StartImmediateRenderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_job_status(
        self, job_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetJobStatusResponse]:
        """
        Fetches detailed job information from S3 storage including job status, template ID, target format, and output details if available.

        Parameters
        ----------
        job_id : str
            ID of the render job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetJobStatusResponse]
            Default Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(job_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetJobStatusResponse,
                    parse_obj_as(
                        type_=GetJobStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

InitializeRenderJobRequestTarget = typing.Union[
    typing.Literal["pdf", "html", "docx", "xlsx", "pptx", "png", "jpg"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawTemplatesClient, RawTemplatesClient
from .types.clone_template_response import CloneTemplateResponse
from .types.delete_template_response import DeleteTemplateResponse
from .types.generate_presigned_get_url_response import GeneratePresignedGetUrlResponse
from .types.generate_template_previews_request_format_opts import GenerateTemplatePreviewsRequestFormatOpts
from .types.generate_template_previews_request_type import GenerateTemplatePreviewsRequestType
from .types.generate_template_previews_response import GenerateTemplatePreviewsResponse
from .types.get_template_index_html_response import GetTemplateIndexHtmlResponse
from .types.initialize_template_creation_response import InitializeTemplateCreationResponse
from .types.save_created_template_request_preview_ids import SaveCreatedTemplateRequestPreviewIds
from .types.save_created_template_request_template_info import SaveCreatedTemplateRequestTemplateInfo
from .types.update_template_request_preview_ids import UpdateTemplateRequestPreviewIds
from .types.update_template_request_template_info import UpdateTemplateRequestTemplateInfo
from .types.update_template_response import UpdateTemplateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTemplatesClient
        """
        return self._raw_client

    def initialize_template_creation(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InitializeTemplateCreationResponse:
        """
        Initializes template creation by generating a unique ID and providing a presigned URL for template ZIP upload. Sets unfinished tag for tracking incomplete templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitializeTemplateCreationResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.initialize_template_creation()
        """
        _response = self._raw_client.initialize_template_creation(request_options=request_options)
        return _response.data

    def save_created_template(
        self,
        template_id: str,
        *,
        template_info: SaveCreatedTemplateRequestTemplateInfo,
        preview_ids: SaveCreatedTemplateRequestPreviewIds,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Finalizes template creation by saving template info to Strapi, copying preview files to permanent storage, and creating template index. Removes unfinished tag upon completion.

        Parameters
        ----------
        template_id : str

        template_info : SaveCreatedTemplateRequestTemplateInfo

        preview_ids : SaveCreatedTemplateRequestPreviewIds

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Pogodoc import PogodocApi
        from Pogodoc.templates import (
            SaveCreatedTemplateRequestPreviewIds,
            SaveCreatedTemplateRequestTemplateInfo,
        )

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.save_created_template(
            template_id="templateId",
            template_info=SaveCreatedTemplateRequestTemplateInfo(
                title="title",
                description="description",
                type="docx",
                sample_data={"key": "value"},
                categories=["invoice"],
            ),
            preview_ids=SaveCreatedTemplateRequestPreviewIds(
                png_job_id="pngJobId",
                pdf_job_id="pdfJobId",
            ),
        )
        """
        _response = self._raw_client.save_created_template(
            template_id, template_info=template_info, preview_ids=preview_ids, request_options=request_options
        )
        return _response.data

    def update_template(
        self,
        template_id: str,
        *,
        template_info: UpdateTemplateRequestTemplateInfo,
        preview_ids: UpdateTemplateRequestPreviewIds,
        content_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateTemplateResponse:
        """
        Updates template content, handles S3 storage cleanup for old content, updates template metadata in Strapi, and manages preview files. Removes unfinished tags after successful update.

        Parameters
        ----------
        template_id : str

        template_info : UpdateTemplateRequestTemplateInfo

        preview_ids : UpdateTemplateRequestPreviewIds

        content_id : str
            ID by which the new template content is saved

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateTemplateResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi
        from Pogodoc.templates import (
            UpdateTemplateRequestPreviewIds,
            UpdateTemplateRequestTemplateInfo,
        )

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.update_template(
            template_id="templateId",
            template_info=UpdateTemplateRequestTemplateInfo(
                title="title",
                description="description",
                type="docx",
                sample_data={"key": "value"},
                categories=["invoice"],
            ),
            preview_ids=UpdateTemplateRequestPreviewIds(
                png_job_id="pngJobId",
                pdf_job_id="pdfJobId",
            ),
            content_id="contentId",
        )
        """
        _response = self._raw_client.update_template(
            template_id,
            template_info=template_info,
            preview_ids=preview_ids,
            content_id=content_id,
            request_options=request_options,
        )
        return _response.data

    def delete_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteTemplateResponse:
        """
        Deletes a template from Strapi and associated S3 storage. Removes all associated files and metadata.

        Parameters
        ----------
        template_id : str
            ID of the template to be deleted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteTemplateResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.delete_template(
            template_id="templateId",
        )
        """
        _response = self._raw_client.delete_template(template_id, request_options=request_options)
        return _response.data

    def extract_template_files(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Extracts contents from an uploaded template ZIP file and stores individual files in the appropriate S3 storage structure.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.extract_template_files(
            template_id="templateId",
        )
        """
        _response = self._raw_client.extract_template_files(template_id, request_options=request_options)
        return _response.data

    def generate_template_previews(
        self,
        template_id: str,
        *,
        type: GenerateTemplatePreviewsRequestType,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        format_opts: typing.Optional[GenerateTemplatePreviewsRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateTemplatePreviewsResponse:
        """
        Creates both PNG and PDF preview files for template visualization. Generates previews in parallel and returns URLs for both formats.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        type : GenerateTemplatePreviewsRequestType
            Type of template to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        format_opts : typing.Optional[GenerateTemplatePreviewsRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GenerateTemplatePreviewsResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.generate_template_previews(
            template_id="templateId",
            type="docx",
            data={"key": "value"},
        )
        """
        _response = self._raw_client.generate_template_previews(
            template_id, type=type, data=data, format_opts=format_opts, request_options=request_options
        )
        return _response.data

    def generate_presigned_get_url(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneratePresignedGetUrlResponse:
        """
        Generates a presigned URL for template access. Used for downloading template files from S3 storage.

        Parameters
        ----------
        template_id : str
            ID of the template that is being downloaded

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneratePresignedGetUrlResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.generate_presigned_get_url(
            template_id="templateId",
        )
        """
        _response = self._raw_client.generate_presigned_get_url(template_id, request_options=request_options)
        return _response.data

    def get_template_index_html(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTemplateIndexHtmlResponse:
        """
        Retrieves the template index.html file from S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTemplateIndexHtmlResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.get_template_index_html(
            template_id="templateId",
        )
        """
        _response = self._raw_client.get_template_index_html(template_id, request_options=request_options)
        return _response.data

    def upload_template_index_html(
        self, template_id: str, *, index_html: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Uploads the template index.html file to S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        index_html : str
            New index.html file of the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.upload_template_index_html(
            template_id="templateId",
            index_html="indexHtml",
        )
        """
        _response = self._raw_client.upload_template_index_html(
            template_id, index_html=index_html, request_options=request_options
        )
        return _response.data

    def clone_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CloneTemplateResponse:
        """
        Creates a new template by duplicating an existing template's content and metadata. Includes copying preview files and template index.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloneTemplateResponse
            Default Response

        Examples
        --------
        from Pogodoc import PogodocApi

        client = PogodocApi(
            token="YOUR_TOKEN",
        )
        client.templates.clone_template(
            template_id="templateId",
        )
        """
        _response = self._raw_client.clone_template(template_id, request_options=request_options)
        return _response.data


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTemplatesClient
        """
        return self._raw_client

    async def initialize_template_creation(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InitializeTemplateCreationResponse:
        """
        Initializes template creation by generating a unique ID and providing a presigned URL for template ZIP upload. Sets unfinished tag for tracking incomplete templates.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitializeTemplateCreationResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.initialize_template_creation()


        asyncio.run(main())
        """
        _response = await self._raw_client.initialize_template_creation(request_options=request_options)
        return _response.data

    async def save_created_template(
        self,
        template_id: str,
        *,
        template_info: SaveCreatedTemplateRequestTemplateInfo,
        preview_ids: SaveCreatedTemplateRequestPreviewIds,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Finalizes template creation by saving template info to Strapi, copying preview files to permanent storage, and creating template index. Removes unfinished tag upon completion.

        Parameters
        ----------
        template_id : str

        template_info : SaveCreatedTemplateRequestTemplateInfo

        preview_ids : SaveCreatedTemplateRequestPreviewIds

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi
        from Pogodoc.templates import (
            SaveCreatedTemplateRequestPreviewIds,
            SaveCreatedTemplateRequestTemplateInfo,
        )

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.save_created_template(
                template_id="templateId",
                template_info=SaveCreatedTemplateRequestTemplateInfo(
                    title="title",
                    description="description",
                    type="docx",
                    sample_data={"key": "value"},
                    categories=["invoice"],
                ),
                preview_ids=SaveCreatedTemplateRequestPreviewIds(
                    png_job_id="pngJobId",
                    pdf_job_id="pdfJobId",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.save_created_template(
            template_id, template_info=template_info, preview_ids=preview_ids, request_options=request_options
        )
        return _response.data

    async def update_template(
        self,
        template_id: str,
        *,
        template_info: UpdateTemplateRequestTemplateInfo,
        preview_ids: UpdateTemplateRequestPreviewIds,
        content_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateTemplateResponse:
        """
        Updates template content, handles S3 storage cleanup for old content, updates template metadata in Strapi, and manages preview files. Removes unfinished tags after successful update.

        Parameters
        ----------
        template_id : str

        template_info : UpdateTemplateRequestTemplateInfo

        preview_ids : UpdateTemplateRequestPreviewIds

        content_id : str
            ID by which the new template content is saved

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateTemplateResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi
        from Pogodoc.templates import (
            UpdateTemplateRequestPreviewIds,
            UpdateTemplateRequestTemplateInfo,
        )

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.update_template(
                template_id="templateId",
                template_info=UpdateTemplateRequestTemplateInfo(
                    title="title",
                    description="description",
                    type="docx",
                    sample_data={"key": "value"},
                    categories=["invoice"],
                ),
                preview_ids=UpdateTemplateRequestPreviewIds(
                    png_job_id="pngJobId",
                    pdf_job_id="pdfJobId",
                ),
                content_id="contentId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_template(
            template_id,
            template_info=template_info,
            preview_ids=preview_ids,
            content_id=content_id,
            request_options=request_options,
        )
        return _response.data

    async def delete_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteTemplateResponse:
        """
        Deletes a template from Strapi and associated S3 storage. Removes all associated files and metadata.

        Parameters
        ----------
        template_id : str
            ID of the template to be deleted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteTemplateResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.delete_template(
                template_id="templateId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_template(template_id, request_options=request_options)
        return _response.data

    async def extract_template_files(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Extracts contents from an uploaded template ZIP file and stores individual files in the appropriate S3 storage structure.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.extract_template_files(
                template_id="templateId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extract_template_files(template_id, request_options=request_options)
        return _response.data

    async def generate_template_previews(
        self,
        template_id: str,
        *,
        type: GenerateTemplatePreviewsRequestType,
        data: typing.Dict[str, typing.Optional[typing.Any]],
        format_opts: typing.Optional[GenerateTemplatePreviewsRequestFormatOpts] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GenerateTemplatePreviewsResponse:
        """
        Creates both PNG and PDF preview files for template visualization. Generates previews in parallel and returns URLs for both formats.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        type : GenerateTemplatePreviewsRequestType
            Type of template to be rendered

        data : typing.Dict[str, typing.Optional[typing.Any]]
            Sample data for the template

        format_opts : typing.Optional[GenerateTemplatePreviewsRequestFormatOpts]
            Format options for the rendered document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GenerateTemplatePreviewsResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.generate_template_previews(
                template_id="templateId",
                type="docx",
                data={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.generate_template_previews(
            template_id, type=type, data=data, format_opts=format_opts, request_options=request_options
        )
        return _response.data

    async def generate_presigned_get_url(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneratePresignedGetUrlResponse:
        """
        Generates a presigned URL for template access. Used for downloading template files from S3 storage.

        Parameters
        ----------
        template_id : str
            ID of the template that is being downloaded

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneratePresignedGetUrlResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.generate_presigned_get_url(
                template_id="templateId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.generate_presigned_get_url(template_id, request_options=request_options)
        return _response.data

    async def get_template_index_html(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTemplateIndexHtmlResponse:
        """
        Retrieves the template index.html file from S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTemplateIndexHtmlResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.get_template_index_html(
                template_id="templateId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_template_index_html(template_id, request_options=request_options)
        return _response.data

    async def upload_template_index_html(
        self, template_id: str, *, index_html: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Uploads the template index.html file to S3 storage. Used for rendering the template in the browser.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        index_html : str
            New index.html file of the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.upload_template_index_html(
                template_id="templateId",
                index_html="indexHtml",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_template_index_html(
            template_id, index_html=index_html, request_options=request_options
        )
        return _response.data

    async def clone_template(
        self, template_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CloneTemplateResponse:
        """
        Creates a new template by duplicating an existing template's content and metadata. Includes copying preview files and template index.

        Parameters
        ----------
        template_id : str
            ID of the template to be used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloneTemplateResponse
            Default Response

        Examples
        --------
        import asyncio

        from Pogodoc import AsyncPogodocApi

        client = AsyncPogodocApi(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.clone_template(
                template_id="templateId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.clone_template(template_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class UpdateTemplateRequestPreviewIds(UniversalBaseModel):
    png_job_id: typing_extensions.Annotated[str, FieldMetadata(alias="pngJobId")] = pydantic.Field()
    """
    ID of the job for the PNG preview
    """

    pdf_job_id: typing_extensions.Annotated[str, FieldMetadata(alias="pdfJobId")] = pydantic.Field()
    """
    ID of the job for the PDF preview
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SaveCreatedTemplateRequestTemplateInfoCategoriesItem = typing.Union[
    typing.Literal["invoice", "mail", "report", "cv", "other"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .initialize_render_job_response_target import InitializeRenderJobResponseTarget


class InitializeRenderJobResponse(UniversalBaseModel):
    job_id: typing_extensions.Annotated[str, FieldMetadata(alias="jobId")] = pydantic.Field()
    """
    ID of the render job
    """

    target: InitializeRenderJobResponseTarget = pydantic.Field()
    """
    Type of output to be rendered
    """

    presigned_data_upload_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="presignedDataUploadUrl")
    ] = pydantic.Field(default=None)
    """
    Presigned URL to upload the data for the render job to S3
    """

    presigned_template_upload_url: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="presignedTemplateUploadUrl")
    ] = pydantic.Field(default=None)
    """
    Presigned URL to upload the template for the render job to S3. Only works with EJS templates
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import httpx
import pydantic
import typing
from pogodoc.client.documents.types import (
    InitializeRenderJobRequestType,
    InitializeRenderJobRequestFormatOpts,
    InitializeRenderJobRequestTarget,
)
from pogodoc.client.core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel

def upload_to_s3_with_url(presigned_url: str, payload, payload_length, content_type: str):
    headers = {
        "Content-Length": str(payload_length),
    }

    if content_type:
        headers["Content-Type"] = content_type

    httpx.put(presigned_url, content=payload, headers=headers)

class RenderConfig(UniversalBaseModel):
    type: InitializeRenderJobRequestType
    target: InitializeRenderJobRequestTarget
    format_opts: typing.Optional[InitializeRenderJobRequestFormatOpts] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
